@@ -445,8 +445,8 @@ MagickExport Image *PingImages(ImageInfo *image_info,const char *filename,
 %
 */
 
-static MagickBooleanType IsCoderAuthorized(const char *coder,
-  const PolicyRights rights,ExceptionInfo *exception)
+static MagickBooleanType IsCoderAuthorized(const char *module,
+  const char *coder,const PolicyRights rights,ExceptionInfo *exception)
 {
   if (IsRightsAuthorized(CoderPolicyDomain,rights,coder) == MagickFalse)
     {
@@ -455,6 +455,13 @@ static MagickBooleanType IsCoderAuthorized(const char *coder,
         "NotAuthorized","`%s'",coder);
       return(MagickFalse);
     }
+  if (IsRightsAuthorized(ModulePolicyDomain,rights,module) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",module);
+      return(MagickFalse);
+    }
   return(MagickTrue);
 }
 
@@ -485,9 +492,9 @@ static void InitializeConstituteInfo(const ImageInfo *image_info,
       constitute_info->delay_flags=ParseGeometry(option,&geometry_info);
       if (constitute_info->delay_flags != NoValue)
         {
-          constitute_info->delay=floor(geometry_info.rho+0.5);
+          constitute_info->delay=(size_t) floor(geometry_info.rho+0.5);
           if ((constitute_info->delay_flags & SigmaValue) != 0)
-            constitute_info->ticks_per_second=CastDoubleToLong(floor(
+            constitute_info->ticks_per_second=CastDoubleToSsizeT(floor(
               geometry_info.sigma+0.5));
         }
     }
@@ -727,15 +734,14 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       /*
         Call appropriate image reader based on image type.
       */
-      if ((magick_info != (const MagickInfo *) NULL) &&
-          (GetMagickDecoderThreadSupport(magick_info) == MagickFalse))
+      if (GetMagickDecoderThreadSupport(magick_info) == MagickFalse)
         LockSemaphoreInfo(magick_info->semaphore);
-      status=IsCoderAuthorized(read_info->magick,ReadPolicyRights,exception);
+      status=IsCoderAuthorized(magick_info->magick_module,read_info->magick,
+        ReadPolicyRights,exception);
       image=(Image *) NULL;
       if (status != MagickFalse)
         image=decoder(read_info,exception);
-      if ((magick_info != (const MagickInfo *) NULL) &&
-          (GetMagickDecoderThreadSupport(magick_info) == MagickFalse))
+      if (GetMagickDecoderThreadSupport(magick_info) == MagickFalse)
         UnlockSemaphoreInfo(magick_info->semaphore);
     }
   else
@@ -745,7 +751,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
-            read_info->magick);
+            read_info->filename);
           if (read_info->temporary != MagickFalse)
             (void) RelinquishUniqueFileResource(read_info->filename);
           read_info=DestroyImageInfo(read_info);
@@ -792,7 +798,8 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       */
       if (GetMagickDecoderThreadSupport(magick_info) == MagickFalse)
         LockSemaphoreInfo(magick_info->semaphore);
-      status=IsCoderAuthorized(read_info->magick,ReadPolicyRights,exception);
+      status=IsCoderAuthorized(magick_info->magick_module,read_info->magick,
+        ReadPolicyRights,exception);
       image=(Image *) NULL;
       if (status != MagickFalse)
         image=(decoder)(read_info,exception);
@@ -843,12 +850,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     const StringInfo
       *profile;
 
-    static const char
-      *source_date_epoch = (const char *) NULL;
-
-    static MagickBooleanType
-      epoch_initialized = MagickFalse;
-
     next->taint=MagickFalse;
     GetPathComponent(magick_filename,MagickPath,magick_path);
     if ((*magick_path == '\0') && (*next->magick == '\0'))
@@ -938,12 +939,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     profile=GetImageProfile(next,"iptc");
     if (profile == (const StringInfo *) NULL)
       profile=GetImageProfile(next,"8bim");
-    if (epoch_initialized == MagickFalse)
-      {
-        source_date_epoch=getenv("SOURCE_DATE_EPOCH");
-        epoch_initialized=MagickTrue;
-      }
-    if (source_date_epoch == (const char *) NULL)
+    if (IsSourceDataEpochSet() == MagickFalse)
       {
         char
           timestamp[MagickTimeExtent];
@@ -1345,14 +1341,13 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       /*
         Call appropriate image writer based on image type.
       */
-      if ((magick_info != (const MagickInfo *) NULL) &&
-          (GetMagickEncoderThreadSupport(magick_info) == MagickFalse))
+      if (GetMagickEncoderThreadSupport(magick_info) == MagickFalse)
         LockSemaphoreInfo(magick_info->semaphore);
-      status=IsCoderAuthorized(write_info->magick,WritePolicyRights,exception);
+      status=IsCoderAuthorized(magick_info->magick_module,write_info->magick,
+        WritePolicyRights,exception);
       if (status != MagickFalse)
         status=encoder(write_info,image,exception);
-      if ((magick_info != (const MagickInfo *) NULL) &&
-          (GetMagickEncoderThreadSupport(magick_info) == MagickFalse))
+      if (GetMagickEncoderThreadSupport(magick_info) == MagickFalse)
         UnlockSemaphoreInfo(magick_info->semaphore);
     }
   else
@@ -1420,8 +1415,8 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
               */
               if (GetMagickEncoderThreadSupport(magick_info) == MagickFalse)
                 LockSemaphoreInfo(magick_info->semaphore);
-              status=IsCoderAuthorized(write_info->magick,WritePolicyRights,
-                exception);
+              status=IsCoderAuthorized(magick_info->magick_module,write_info->magick,
+                WritePolicyRights,exception);
               if (status != MagickFalse)
                 status=encoder(write_info,image,exception);
               if (GetMagickEncoderThreadSupport(magick_info) == MagickFalse)
