@@ -53,13 +53,14 @@
 #include "MagickCore/list.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
 #include "MagickCore/option.h"
 #include "MagickCore/property.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
-#include "MagickCore/module.h"
+#include "MagickCore/thread-private.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/xwindow.h"
 #include "MagickCore/xwindow-private.h"
@@ -171,14 +172,12 @@ typedef struct _AshlarInfo
 {
   size_t
     width,
-    height;
+    height,
+    number_nodes;
 
   ssize_t
     align;
 
-  size_t
-    number_nodes;
-
   MagickBooleanType
     best_fit;
 
@@ -458,6 +457,9 @@ static inline MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   /*
     Pack tiles so they fit the canvas with minimum excess.
   */
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status)
+#endif
   for (i=0; i < (ssize_t) number_tiles; i++)
     tiles[i].order=(i);
   qsort((void *) tiles,number_tiles,sizeof(*tiles),CompareTileHeight);
@@ -483,6 +485,9 @@ static inline MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   }
   qsort((void *) tiles,number_tiles,sizeof(*tiles),RestoreTileOrder);
   status=MagickTrue;
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status)
+#endif
   for (i=0; i < (ssize_t) number_tiles; i++)
   {
     tiles[i].order=(ssize_t) ((tiles[i].x != (ssize_t) MAGICK_SSIZE_MAX) ||
@@ -541,8 +546,8 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
       }
       geometry.width=(size_t) geometry.width/7;
       geometry.height=(size_t) geometry.height/7;
-      geometry.x=(ssize_t) pow((double) geometry.width,0.25);
-      geometry.y=(ssize_t) pow((double) geometry.height,0.25);
+      geometry.x=(ssize_t) pow((double) geometry.width,0.4);
+      geometry.y=(ssize_t) pow((double) geometry.height,0.4);
       image_info->extract=AcquireString("");
       if (image_info->extract != (char *) NULL)
         (void) FormatLocaleString(image_info->extract,MagickPathExtent,
@@ -618,11 +623,17 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
   value=GetImageOption(image_info,"label");
   extent.width=0;
   extent.height=0;
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status,extent) \
+    magick_number_threads(image,image,n,1)
+#endif
   for (i=0; i < n; i++)
   {
     Image
       *tile_image;
 
+    if (status == MagickFalse)
+      continue;
     if ((tiles[i].x == (ssize_t) MAGICK_SSIZE_MAX) ||
         (tiles[i].y == (ssize_t) MAGICK_SSIZE_MAX))
       continue;
@@ -630,9 +641,14 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
       ((ssize_t) tiles[i].width-2*geometry.x),(size_t)
       ((ssize_t) tiles[i].height-2*geometry.y),image->filter,exception);
     if (tile_image == (Image *) NULL)
-      continue;
-    (void) CompositeImage(ashlar_image,tile_image,image->compose,MagickTrue,
+      {
+        status=MagickFalse;
+        continue;
+      }
+    status=CompositeImage(ashlar_image,tile_image,image->compose,MagickTrue,
       tiles[i].x+geometry.x,tiles[i].y+geometry.y,exception);
+    if (status == MagickFalse)
+      continue;
     if (value != (const char *) NULL)
       {
         char
@@ -647,24 +663,33 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
         if (label != (const char *) NULL)
           {
             (void) CloneString(&draw_info->text,label);
-            draw_info->pointsize=1.8*geometry.y;
+            label=DestroyString(label);
             (void) FormatLocaleString(offset,MagickPathExtent,"%+g%+g",(double)
-              tiles[i].x+geometry.x,(double) tiles[i].height+tiles[i].y+
-              geometry.y/2.0);
+              tiles[i].x+geometry.x,(double) tiles[i].height+tiles[i].y-
+              geometry.y/2.0+4);
             (void) CloneString(&draw_info->geometry,offset);
-            (void) AnnotateImage(ashlar_image,draw_info,exception);
+            status=AnnotateImage(ashlar_image,draw_info,exception);
           }
       }
-    if (((ssize_t) tiles[i].width+tiles[i].x) > (ssize_t) extent.width)
-      extent.width=(size_t) ((ssize_t) tiles[i].width+tiles[i].x);
-    if (((ssize_t) tiles[i].height+tiles[i].y+geometry.y+2) > (ssize_t) extent.height)
-      extent.height=(size_t) ((ssize_t) tiles[i].height+tiles[i].y+
-        geometry.y+2);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+    #pragma omp critical
+#endif
+    {
+      if (((ssize_t) tiles[i].width+tiles[i].x) > (ssize_t) extent.width)
+        extent.width=(size_t) ((ssize_t) tiles[i].width+tiles[i].x);
+      if (((ssize_t) tiles[i].height+tiles[i].y+geometry.y+2) > (ssize_t) extent.height)
+        extent.height=(size_t) ((ssize_t) tiles[i].height+tiles[i].y+
+          geometry.y+2);
+    }
     tile_image=DestroyImage(tile_image);
   }
+  if (image_info->extract != (char *) NULL)
+    (void) ParseAbsoluteGeometry(image_info->extract,&extent);
   (void) SetImageExtent(ashlar_image,extent.width,extent.height,exception);
   nodes=(NodeInfo *) RelinquishMagickMemory(nodes);
   tiles=(CanvasInfo *) RelinquishMagickMemory(tiles);
+  if (status == MagickFalse)
+    ashlar_image=DestroyImage(ashlar_image);
   return(ashlar_image);
 }
 
