@@ -422,6 +422,9 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   else
     {
 #if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,17,0)
+      enum heif_item_property_type
+        type = heif_item_property_type_invalid;
+
       heif_item_id
         item_id;
 
@@ -435,6 +438,9 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
       count=heif_item_get_transformation_properties(heif_context,item_id,
         transforms,1);
       if (count == 1)
+        type=heif_item_get_property_type(heif_context,item_id,transforms[0]);
+      if (count == 1 && ((type == heif_item_property_type_transform_mirror) ||
+          (type == heif_item_property_type_transform_rotation)))
         {
           enum heif_transform_mirror_direction
             mirror;
